@@ -5,12 +5,12 @@ mod size_info;
 
 pub use page_no::*;
 
-use page::Page;
 use file::FileExt;
-use std::fs::File;
-use std::path::Path;
+use page::Page;
 use size_info::SizeInfo;
+use std::fs::File;
 use std::io::{Error, ErrorKind, Result};
+use std::path::Path;
 use std::sync::atomic::{AtomicU32, Ordering};
 
 use page_lock::PageLocker;
@@ -51,6 +51,7 @@ impl<K: PageNo, const NBYTES: usize> Pages<K, NBYTES> {
         // Is new file? set the length to 1.
         if file_len == 0 {
             len = 1;
+            file.set_len(NBYTES as u64)?;
         }
         // Exist File?  Check size info.
         else {
@@ -130,14 +131,29 @@ impl<K: PageNo, const NBYTES: usize> Pages<K, NBYTES> {
         }
     }
 
+    /// Increase page number by `count`, and return the new length.
+    pub async fn alloc(&self, count: u32) -> Result<u32> {
+        let old_len = self.len.fetch_add(count, Ordering::SeqCst);
+        let new_len = old_len + count;
+        let file = self.file;
+        unsafe {
+            spawn_blocking(move || {
+                file.set_len(NBYTES as u64 * new_len as u64)?;
+                Ok(new_len)
+            })
+            .await
+            .unwrap_unchecked()
+        }
+    }
+
     pub async fn create(&self, buf: [u8; NBYTES]) -> Result<usize> {
-        let num = self.len.fetch_add(1, Ordering::Relaxed);
+        let num = self.len.fetch_add(1, Ordering::SeqCst);
         self.write(PageNo::new(num), buf).await
     }
 
     #[allow(clippy::len_without_is_empty)]
     pub fn len(&self) -> u32 {
-        self.len.load(Ordering::Relaxed)
+        self.len.load(Ordering::SeqCst)
     }
 
     pub fn inner(&self) -> &'static File {
@@ -157,7 +173,7 @@ impl<K: PageNo, const NBYTES: usize> Drop for Pages<K, NBYTES> {
 mod tests {
     use super::*;
 
-    async fn init() -> Result<usize> {
+    async fn read_write() -> Result<()> {
         let pages = Pages::<u16, 64>::open("test.db")?;
 
         assert_eq!(pages.len(), 1);
@@ -166,20 +182,27 @@ mod tests {
 
         let mut page = pages.goto(1).await?;
         page.buf = [1; 64];
-        page.write().await
-    }
-    async fn create_page() -> Result<()> {
-        let pages = Pages::<u16, 64>::open("test.db")?;
-        assert_eq!(pages.len(), 2);
+        page.write().await?;
+
         assert_eq!(pages.read(1).await?, [1; 64]);
         pages.create([0; 64]).await?;
 
         assert_eq!(pages.len(), 3);
 
         pages.write(2, [2; 64]).await?;
-        assert_eq!(pages.len(), 3);
         assert_eq!(pages.read(2).await?, [2; 64]);
-        Ok(())
+        std::fs::remove_file("test.db")
+    }
+
+    async fn alloc_page() -> Result<()> {
+        let pages = Pages::<u16, 64>::open("test2.db")?;
+
+        pages.create([1; 64]).await?;
+        assert_eq!(pages.len(), 2);
+        pages.alloc(1).await?;
+        assert_eq!(pages.len(), 3);
+
+        std::fs::remove_file("test2.db")
     }
 
     #[test]
@@ -189,9 +212,8 @@ mod tests {
             .build()
             .expect("Failed building the Runtime")
             .block_on(async {
-                init().await.expect("Failed to Initialize");
-                create_page().await.expect("Failed creating page");
-                std::fs::remove_file("test.db").unwrap()
+                read_write().await.unwrap();
+                alloc_page().await.unwrap();
             });
     }
 }
